<?php

namespace TDevAgency\FilesManager\Helpers;

use claviska\SimpleImage;
use Exception;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Facades\Storage;
use TDevAgency\FilesManager\ResourceModels\TdfmFile;

class GenerateSizesHelper
{
    /**
     * @param TdfmFile $file
     * @return void
     * @throws Exception
     */
    public function handle(TdfmFile $file, array $sizes = []): void
    {
        $name = preg_replace('/(\w+)(.)(\w+)/', '${1}', $file->name);
        $path = Storage::disk('tdfm_original')->path($file->name);
        $originalExtension = pathinfo($file->name, PATHINFO_EXTENSION);
        if (!File::exists($path) || !preg_match('/^image\/(jpe?g|gif|png|webp)+$/', $file->file_mime)) {
            return;
        }

        if (empty($sizes)) {
            $requiredSizes = Config::get('tdfm.sizes');
        } else {
            $requiredSizes = collect(Config::get('tdfm.sizes'))->filter(function ($item, $key) use ($sizes) {
                return in_array($key, $sizes);
            })->toArray();
        }

        foreach ($requiredSizes as $key => $size) {
            $image = (new SimpleImage())->fromFile($path);

            $width = min($image->getWidth(), $size);
            if ($key !== 'xs') {
                $image->resize($width);
            } else {
                $image->thumbnail($width, $width);
            }
            $image->autoOrient();

            Storage::disk('tdfm_generated')->put(
                sprintf('%s_new/%s/%s.%s', $file->id, $key, $name, $originalExtension),
                $image->toString($file->file_mime, Config::get('tdfm.quality'))
            );

            Storage::disk('tdfm_generated')->put(
                sprintf('%s_new/%s/%s.%s', $file->id, $key, $name, $originalExtension === 'webp' ? 'png' : 'webp'),
                $image->toString('image/webp', Config::get('tdfm.quality'))
            );
        }

        if (File::isDirectory(Storage::disk('tdfm_generated')->path($file->id))) {
            File::cleanDirectory(Storage::disk('tdfm_generated')->path($file->id));
        }

        File::move(
            Storage::disk('tdfm_generated')->path($file->id . '_new'),
            Storage::disk('tdfm_generated')->path($file->id)
        );

        app(CacheHelper::class)->removeFileData($file->id);
    }
}
