<?php

namespace TDevAgency\FilesManager\Helpers;

use Illuminate\Support\Facades\Config;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Facades\Storage;
use TDevAgency\FilesManager\ResourceModels\TdfmFile;

class SymlinkHelper
{
    /** @var GenerateSizesHelper */
    private $generateSizesHelper;

    public function __construct(GenerateSizesHelper $generateSizesHelper)
    {
        $this->generateSizesHelper = $generateSizesHelper;
    }

    public function symlinkFile(TdfmFile $file, bool $makeSymlink = false): void
    {
        $sizes = array_keys(Config::get('tdfm.sizes'));

        if (!File::isDirectory(public_path('media/' . $file->id))) {
            File::makeDirectory(public_path('media/' . $file->id));
            chmod(public_path('media/' . $file->id), 0775);
        } else {
            File::cleanDirectory(public_path('media/' . $file->id));
        }

        $pathInfo = pathinfo($file->original_name);
        $pathInfoCached = pathinfo($file->name);
        if (!File::exists(public_path('media/' . $file->id . '/' . $file->original_name))) {
            File::link(
                Storage::disk('tdfm_original')->path($file->name),
                public_path('media/' . $file->id . '/' . $file->original_name)
            );
        }
        if (Storage::disk('tdfm_generated')->exists($file->id)) {
            $webpName = $pathInfoCached['filename'] . '.webp';
            $newWebpName = $pathInfo['filename'] . '.webp';

            foreach ($sizes as $size) {
                if (!Storage::disk('tdfm_generated')->exists($file->id . '/' . $size)) {
                    $this->generateSizesHelper->handle($file, [$size]);
                }
                if (!File::isDirectory(public_path('media/' . $file->id . '/' . $size))) {
                    File::makeDirectory(public_path('media/' . $file->id . '/' . $size));
                    chmod(public_path('media/' . $file->id . '/' . $size), 0775);
                }

                if (
                    !File::exists(
                        public_path('media/' . $file->id . '/' . $size . '/' . $file->original_name)
                    )
                ) {
                    $this->makeCopySymlink(
                        Storage::disk('tdfm_generated')->path($file->id . '/' . $size . '/' . $file->name),
                        public_path('media/' . $file->id . '/' . $size . '/' . $file->original_name),
                        $makeSymlink
                    );
                }
                if (Storage::disk('tdfm_generated')->exists($file->id . '/' . $size . '/' . $webpName)) {
                    if (!File::exists(public_path('media/' . $file->id . '/' . $size . '/' . $newWebpName))) {
                        $this->makeCopySymlink(
                            Storage::disk('tdfm_generated')->path($file->id . '/' . $size . '/' . $webpName),
                            public_path('media/' . $file->id . '/' . $size . '/' . $newWebpName),
                            $makeSymlink
                        );
                    }
                }
            }
        }

        app(CacheHelper::class)->removeFileData($file->id);
    }

    private function makeCopySymlink(string $originalPath, string $targetPath, bool $link = false): void
    {
        $link ? File::link(
            $originalPath,
            $targetPath
        ) : File::copy(
            $originalPath,
            $targetPath
        );
    }
}
