<?php

namespace TDevAgency\FilesManager\Jobs;

use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Queue\SerializesModels;
use TDevAgency\FilesManager\Helpers\CacheHelper;
use TDevAgency\FilesManager\Helpers\SymlinkHelper;
use TDevAgency\FilesManager\ResourceModels\TdfmFile;

class FilesManagerSymlinkJob implements ShouldQueue
{
    use Dispatchable;
    use InteractsWithQueue;
    use Queueable;
    use SerializesModels;

    /** @var TdfmFile $file */
    private $file;

    /** @var bool */
    private $makeAsLink;

    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct(TdfmFile $file, bool $makeAsLink = false)
    {
        $this->file = $file;
        $this->makeAsLink = $makeAsLink;
    }

    /**
     * Execute the job.
     *
     * @return void
     */
    public function handle(): void
    {
        app(SymlinkHelper::class)->symlinkFile($this->file, $this->makeAsLink);

        app(CacheHelper::class)->removeFileData($this->file->id);
    }
}
