<?php

namespace TDevAgency\FilesManager\ResourceModels;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\Schema;
use Kyslik\ColumnSortable\Sortable;
use TDevAgency\FilesManager\MediaHelper;

class TdfmFile extends Model
{
    use Sortable;
    use MediaHelper;

    protected $table = 'files_manager';

    public $sortable = [
        'id',
        'original_name',
        'updated_at',
        'file_size'
    ];
    protected $fillable = [
        'original_name',
        'name',
        'file_size',
        'file_mime',
    ];
    protected $appends = [
        'url',
        'generated',
        'webp'
    ];

    /**
     * @param $table
     * @return TdfmFile
     */
    public function setTable($table): TdfmFile
    {
        return parent::setTable(Schema::hasTable('tdfm_files') ? 'tdfm_files' : 'files_manager');
    }

    public function getUrlAttribute(): ?string
    {
        if (isset($this->attributes['name'])) {
            return $this->attributes['url'] = $this->getImage($this->id);
        }
        return $this->attributes['url'] = null;
    }

    public function getGeneratedAttribute()
    {
        $generated = [];
        foreach (config('tdfm.sizes') as $size => $value) {
            $generated[$size] = $this->getImage($this->id, $size);
        }
        return $this->attributes['generated'] = $generated;
    }

    public function getWebpAttribute()
    {
        if (isset($this->attributes['name']) && config('tdfm.generate_webp')) {
            $generated = [];
            foreach (config('tdfm.sizes') as $size => $value) {
                $generated[$size]  = $this->getImage($this->id, $size, true);
            }
            return $this->attributes['generated'] = $generated;
        }
    }
}
